@@ -28,16 +28,20 @@ digits = "1234567890"
 hexdigits = digits + "ABCDEF"
 
 languages = None 
-with open(str(_base_path / "data" / "languages.json"), "r") as f:
+with (_base_path / "data" / "languages.json").open("r") as f:
     languages = json.loads(f.read())
 
-class Captcha:
-    @staticmethod
-    def de_json(json_str):
-        if json_str: return json.loads(json_str)
-        return None
+class Captcha(types.JsonDeserializable, types.JsonSerializable):
+    @classmethod
+    def de_json(cls, json_str):
+        if not json_str: return None
+        obj = json.loads(json_str)
+        obj['chat'] = types.Chat(**obj['chat'])
+        obj['user'] = types.User(**obj['user'])
+        return cls(**obj)
 
-    def __init__(self, chat: types.Chat, user: types.User, language: str, timeout: float, only_digits: bool, add_noise: bool, bot: TeleBot=None, **kwargs) -> None:
+    def __init__(self, chat: types.Chat, user: types.User, language: str, timeout: float, 
+            only_digits: bool, add_noise: bool, bot: TeleBot=None, **kwargs) -> None:
         """
         The Captcha object. Do not call this function yourself. 
         Use `captcha_manager.send_random_captcha(...)` instead
@@ -45,8 +49,8 @@ class Captcha:
         self._solved = False
         if not bot:
             # Loaded from file
-            self.chat = types.Chat(**chat)
-            self.user = types.User(**user)
+            self.chat = chat
+            self.user = user
             self.language = language
             self.previous_tries = kwargs["previous_tries"]
             self.correct_code = kwargs["correct_code"]
@@ -67,7 +71,7 @@ class Captcha:
 
         else:
             # Initialized by `CaptchaManager.send_random_captcha()`
-            self._captcha_id = f"{CaptchaManager._bot_id}|{chat.id}|{user.id}"
+            self._captcha_id = f"{CaptchaManager._bot_id}={chat.id}={user.id}"
             self._timeout_thread = None
             self._timeout = timeout
             self._only_digits = only_digits
@@ -135,8 +139,7 @@ class Captcha:
             "add_noise": self._add_noise,
             "previous_tries": self.previous_tries
         }
-        return json.dumps(json_dict, ensure_ascii=False, 
-            escape_forward_slashes=False, encode_html_chars=False)
+        return json.dumps(json_dict)
 
     def _continue_timeout(self):
         if self._timeout and CaptchaManager._handlers["on_timeout"]:
@@ -145,7 +148,8 @@ class Captcha:
             if now >= exec_at:
                 self._timeout_thread = Thread(target=CaptchaManager._handlers["on_timeout"], args=[self])
             else:
-                self._timeout_thread = Timer(interval=exec_at-now, function=CaptchaManager._handlers["on_timeout"], args=[self])
+                self._timeout_thread = Timer(interval=exec_at-now, 
+                    function=CaptchaManager._handlers["on_timeout"], args=[self])
             self._timeout_thread.start()
 
     def _refresh(self, bot: TeleBot, only_digits=False, add_noise=True, timeout=None) -> None:
@@ -171,7 +175,7 @@ class Captcha:
             os.mkdir(_captcha_saves)
         filename = self._captcha_id + ".json"
         filepath = _captcha_saves / filename
-        with filepath.open("w") as f:
+        with open(filepath, "w+",encoding="utf8") as f:
             f.write(self.to_json())
     
     def _delete_file(self):
@@ -181,7 +185,7 @@ class Captcha:
             filepath.unlink()
 
     def _update(self, bot: TeleBot, callback: types.CallbackQuery):
-        btn = callback.data.split("|")[2]
+        btn = callback.data.split("=")[2]
         if (btn == "BACK"):
             self.users_code = self.users_code[:-1]
         else:
@@ -242,15 +246,15 @@ class CaptchaManager:
             saved_captchas = os.listdir(_captcha_saves)
             for f in saved_captchas:
                 if f.endswith(".json") and not f.startswith("."):
-                    if f.startswith(f"{self._bot_id}|"):
-                        json_dict = None
+                    if f.startswith(f"{self._bot_id}="):
                         filepath = _captcha_saves / f
                         with filepath.open("r") as f:
-                            json_dict = Captcha.de_json(f.read())
-                        captcha = Captcha(**json_dict)
-                        self.captchas[captcha._captcha_id] = captcha
+                            json_str = f.read()
+                            captcha = Captcha.de_json(json_str)
+                            self.captchas[captcha._captcha_id] = captcha
 
-    def send_random_captcha(self, bot: TeleBot, chat: types.Chat, user: types.User, language: str=None, only_digits: bool=False, add_noise: bool=True, timeout: float=None) -> Captcha:
+    def send_random_captcha(self, bot: TeleBot, chat: types.Chat, user: types.User, language: str=None, 
+            only_digits: bool=False, add_noise: bool=True, timeout: float=None) -> Captcha:
         """
         sends a randomly generated captcha into your chat.
         :param bot: your TeleBot instance
@@ -327,10 +331,10 @@ class CaptchaManager:
         :param bot: your TeleBot instance
         :param callback: the CallbackQuery
         """
-        if not callback.data.startswith("?cap|"): return
+        if not callback.data.startswith("?cap="): return
 
-        user_id, btn = int(callback.data.split("|")[1]), callback.data.split("|")[2]
-        captcha_id = f"{self.__class__._bot_id}|{callback.message.chat.id}|{user_id}"
+        user_id, btn = int(callback.data.split("=")[1]), callback.data.split("=")[2]
+        captcha_id = f"{self.__class__._bot_id}={callback.message.chat.id}={user_id}"
         captcha: Captcha = self.captchas[captcha_id]
 
         if captcha.user.id != callback.from_user.id:
@@ -338,13 +342,19 @@ class CaptchaManager:
             return
         
         if btn == "OK":
-            self._check_captcha(callback)
+            user_id = int(callback.data.split("=")[1])
+            captcha_id = f"{self.__class__._bot_id}={callback.message.chat.id}={user_id}"
+            if (captcha_id in self.captchas):
+                self._check_captcha(captcha_id)
         else:
             captcha._update(bot, callback)
         
         bot.answer_callback_query(callback.id)
     
-    def refresh_captcha(self, bot: TeleBot, captcha: Captcha, only_digits=False, add_noise=True, timeout: float=None) -> None:
+    def refresh_captcha(self, bot: TeleBot, captcha: Captcha, 
+            only_digits=False, add_noise=True, timeout: float=None) -> None:
+        if captcha._timeout_thread is not None:
+            captcha._timeout_thread.cancel()
         timeout = timeout or self.default_timeout
         captcha._refresh(bot, only_digits, add_noise, timeout)
         if timeout:
@@ -357,7 +367,7 @@ class CaptchaManager:
         captcha._solved = False
 
     def delete_captcha(self, bot: TeleBot, captcha: Captcha) -> None:
-        self.captchas.pop(captcha._captcha_id)
+        #self.captchas.pop(captcha._captcha_id)
         captcha._delete_file()
         try: bot.delete_message(captcha.chat.id, captcha.message_id)
         except: pass
@@ -442,31 +452,28 @@ class CaptchaManager:
                 captcha._continue_timeout()
         return wrapper
 
-    def _check_captcha(self, callback: types.CallbackQuery):
-        user_id = int(callback.data.split("|")[1])
-        captcha_id = f"{self.__class__._bot_id}|{callback.message.chat.id}|{user_id}"
-        if (captcha_id in self.captchas):
-            captcha: Captcha = self.captchas[captcha_id]
-            is_correct = captcha.users_code == captcha.correct_code
-            if (captcha._timeout_thread):
-                captcha._timeout_thread.cancel()
-            captcha.previous_tries += 1
-            if is_correct:
-                if (self._handlers["on_correct"]):
-                    self._handlers["on_correct"](captcha)
-            else:
-                if (self._handlers["on_not_correct"]):
-                    self._handlers["on_not_correct"](captcha)
-            captcha._solved = True
+    def _check_captcha(self, captcha: Captcha):
+        is_correct = captcha.users_code == captcha.correct_code
+        if (captcha._timeout_thread):
+            captcha._timeout_thread.cancel()
+            captcha._timeout_thread = None
+        captcha.previous_tries += 1
+        if is_correct:
+            if (self._handlers["on_correct"]):
+                self._handlers["on_correct"](captcha)
+        else:
+            if (self._handlers["on_not_correct"]):
+                self._handlers["on_not_correct"](captcha)
+        captcha._solved = True
 
 
 def _code_input_markup(user_id: int, language: str, only_digits: bool) -> types.InlineKeyboardMarkup:
     values = {}
     for char in (digits if only_digits else hexdigits):
-        values[char] = {"callback_data": f"?cap|{user_id}|{char}"}
+        values[char] = {"callback_data": f"?cap={user_id}={char}"}
     return _quick_markup({**values,
-        languages[language]["back"]: {"callback_data": f"?cap|{user_id}|BACK"},
-        languages[language]["submit"]: {"callback_data": f"?cap|{user_id}|OK"}
+        languages[language]["back"]: {"callback_data": f"?cap={user_id}=BACK"},
+        languages[language]["submit"]: {"callback_data": f"?cap={user_id}=OK"}
     }, 5 if only_digits else 4)
   
 def _quick_markup(values, row_width=4) -> types.InlineKeyboardMarkup:
